"use strict"

function Player (game) {
   var that = this;

   // Assert ourselves as this game's player.
   that.game = game;
   game.player = that;

   // Functions.
   that.GetGame = function () {
      return that.game;
   };
   that.Purchase = function (sItem, quantity) {
      // Get our own items and the store items.
      var pItem = that.GetItem (sItem.name);
      if (pItem == null) {
         console.error ("No item '" + sItem.name + "' for player.");
         return false;
      }

      // Don't allow if we don't have enough money.
      var total = quantity * sItem.price;
      var money = that.GetMoney ();
      if (total > money) {
         console.error ("Not enough money to purchase " +
            pItem.DisplayName (quantity) +
            " ($" + money + " - $" + total + ")");
         return false;
      }

      // Modify money and give us more of the item.
      console.log ("Purchased " + pItem.DisplayName (quantity) +
                   " for $" + total + ".");
      that.ChangeMoney (-total);
      that.ChangeItem (pItem, quantity);
      return true;
   };
   that.GetItem = function (name) {
      if (!(name in that.items))
         return null;
      return that.items[name];
   };
   that.GetStat = function (name) {
      if (!(name in that.stats))
         return null;
      return that.stats[name];
   };
   that.ChangeItem = function (name, amount) {
      var item;
      if (name instanceof Item)
         item = name;
      else
         item = that.GetItem (name);
      if (item == null)
         return false;
      item.Change (amount);
      return true;
   };
   that.GetMoney = function () {
      return that.GetItem ("money").quantity;
   }
   that.ChangeMoney = function (amount) {
      return that.ChangeItem ("money", amount);
   }
   that.LogItems = function () {
      console.log ("Your items:");
      for (var i in that.items) {
         var item = that.items[i];
         console.log ("   " + item.DisplayName ());
      }
      console.log ("");
   };
   that.LogStats = function () {
      console.log ("Your stats:");
      for (var i in that.stats) {
         var stat = that.stats[i];
         console.log ("   " + stat.DisplayName ());
      }
      console.log ("");
   };
   that.Tick = function (hours, ff) {
      if (ff >= 0.00)
         that.MoveTowardsWaypoint (hours);
      if (ff == 0.00) {
         that.DrainStatFromItem ("energy", "coffee", 4.17, hours, 10.00);
         that.DrainStatFromItem ("energy", null,     null, hours,  5.00);
         that.DrainStatFromItem ("morale", "beer",   2.00, hours, 10.00);
      }
      if (ff <= 0.00) {
         that.DrainStatFromItem ("health", "food",   3.42, hours, 10.00);
         if (that.game.IsRaining ())
            that.DrainStatFromItem ("health", "raingear", null, hours, 40.00);
      }
   };
   that.DrainStatFromItem = function (statName, itemName, rate, hours, scale) {
      var stat = that.GetStat (statName);
      var item = that.GetItem (itemName);
      var quantity = item == null ? 0.00 : item.quantity;

      // If we have none of that item, we automatically lose from our stat.
      if (quantity == 0) {
         stat.Change (-hours * scale);
         return;
      }

      // If there's no consumption rate, do nothing if we have that item.
      if (rate == null)
         return;

      // If we have enough of that item to reduce it, do it now.
      var loss = hours / rate;
      if (loss < quantity) {
         item.Change (-loss);
         return;
      }

      // We're losing more than what we have - do fancy math.
      var partialHours = (loss - quantity) * rate;
      if (item != null)
         item.Change (-quantity);
      stat.Change (-partialHours * scale);
   }
   that.SetPosition = function (x, y) {
      that.x = x;
      that.y = y;
   }
   that.UseMap = function (map) {
      // Position ourselves and set up waypoints.
      that.SetPosition (map.landmarks[0].x, map.landmarks[0].y);
      that.NextWaypoint ();
      that.waypoint = map.waypoints[1];
      that.landmark = that.waypoint.NextLandmark ();

      if (map.waypoints[0] && map.waypoints[0].landmark)
         that.game.AtLandmark (map.waypoints[0].landmark);
   }
   that.MoveTowardsWaypoint = function (hours) {
      if (that.waypoint == null)
         return;
      var xTarget = that.waypoint.x;
      var yTarget = that.waypoint.y;
      var xOld    = that.x;
      var yOld    = that.y;

      // Trigonometry!
      var theta = Math.atan2 (yTarget - that.y, xTarget - that.x);
      that.x += Math.cos (theta) * hours * gMapSpeed;
      that.y += Math.sin (theta) * hours * gMapSpeed;

      // Check to see if we've surpassed the waypoint.
      if ((xOld < that.x && that.x >= xTarget) ||
          (xOld > that.x && that.x <= xTarget) ||
          (yOld < that.y && that.y >= yTarget) ||
          (yOld > that.y && that.y <= yTarget)) {
         // Put us there.
         that.x = xTarget;
         that.y = yTarget;

         // Are we at a landmark now?
         if (that.waypoint.landmark)
            that.game.AtLandmark (that.waypoint.landmark);

         // Start moving to the next waypoint.
         that.NextWaypoint ();
      }
   }
   that.NextWaypoint = function () {
      var index = (that.waypoint == null) ? 1 : that.waypoint.index + 1;
      if (index >= that.game.map.waypoints.length) {
         that.game.Victory (null);
         that.waypoint = null;
         that.landmark = null;
      }
      else {
         that.waypoint = that.game.map.waypoints[index];
         that.landmark = that.waypoint.NextLandmark ();
      }
      return that.waypoint;
   }
   that.DrawStatus = function (x, y) {
      gImageSets["status"].images[0].Draw (x, y);

      var xItem = 28;
      for (var i in that.items) {
         var item = that.items[i];
         item.Draw (x + xItem, y + 102, 0.50, {
            hAlign: 0.50,
            vAlign: 0.50
         });
         xItem += 27;
      }
      var yStat = 16;
      for (var i in that.stats) {
         var stat = that.stats[i];
         stat.Draw (x + 41, y + yStat);
         yStat += 20;
      }
   }

   // Add to our game.
   that.game   = game;
   game.player = that;

   // Basic initialization.
   that.Name     = "Nameless";
   that.x        = 80;
   that.y        = 64;
   that.waypoint = null;
   that.landmark = null;
   that.stats    = [];
   that.items    = [];

   // Initialize our items.
   new Item (that, "money",    100,  20.00, 0, -1,
      "dollar", "dollars", "$");
   new Item (that, "beer",     0.00,  1.00,  2,  2,
      "pint of beer",  "pints of beer", null,
      "With so many small,\n" +
      "local breweries in\n" +
      "Portland, keeping your\n" +
      "morale up has never\n" +
      "been easier!");
   new Item (that, "coffee",   0.00,  1.00,  0,  0,
      "cup of coffee", "cups of coffee", null,
      "Coffee is a necessary\n" +
      "part of every Port-\n" +
      "lander's day. With\n" +
      "six months of rain, we\n" +
      "gotta have something\n" +
      "to give us energy!");
   new Item (that, "food",     0.00,  1.00, -1,  1,
      "meal", "meals", null,
      "Portland is one of the\n" +
      "best food cities in the\n" +
      "United States! And, of\n" +
      "course, if you don't\n" +
      "eat, you'll die.");
   new Item (that, "raingear", 1.00,  1.00,  0,  0,
      "piece of rain gear", "pieces of rain gear", null,
      "It's wet to go without.\n" +
      "Take this!");

   // Initialize our stats.
   new Stat (that, "morale", "#0000ff", 50,
      "\"They said Portland was weird, but this is nuts.\n" +
      "I'm not nearly drunk enough to handle this.\"");
   new Stat (that, "energy", "#a0a000", 100,
      "\"Too tired.  Need coffee.\n" +
      "Going home.\"");
   new Stat (that, "health", "#008000", 100,
      "\"Feeling sick - time to bail.\n" +
      "Note to self: Buy a coat, eat food.\"");
}
